/**
 * OW2 Util
 * Copyright (C) 2008 Bull S.A.S.
 * Contact: easybeans@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 *
 * --------------------------------------------------------------------------
 * $Id: WarExternLibraries.java 3253 2008-05-16 12:47:44Z gaellalire $
 * --------------------------------------------------------------------------
 */

package org.ow2.util.ee.metadata.war.impl;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;

import org.ow2.util.archive.api.IArchive;
import org.ow2.util.ee.metadata.war.api.IWarClassMetadata;
import org.ow2.util.ee.metadata.war.api.IWarDeployableMetadata;
import org.ow2.util.ee.metadata.war.impl.configurator.WarDeployableMetadataConfigurator;
import org.ow2.util.log.Log;
import org.ow2.util.log.LogFactory;
import org.ow2.util.scan.api.ArchiveScanner;
import org.ow2.util.scan.api.ArchiveScannerFactory;
import org.ow2.util.scan.api.ScanException;

/**
 * Default implementation.
 * @author Gael Lalire
 */
public class WarExternLibraries {

    /**
     * A logger.
     */
    private static final Log LOGGER = LogFactory.getLog(WarExternLibraries.class);

    /**
     * Associate a class name to its metadata.
     */
    private Map<String, IWarClassMetadata> classMetadata;

    /**
     * Associate an archive scanner to its metadata.
     */
    private Map<ArchiveScanner, IWarDeployableMetadata> scanners;

    /**
     * Construct an extern library.
     * @param archiveScannerFactory the archive scanner factory
     * @param archiveList the archive list
     */
    public WarExternLibraries(final ArchiveScannerFactory archiveScannerFactory, final List<IArchive> archiveList) {
        classMetadata = new HashMap<String, IWarClassMetadata>();
        scanners = new HashMap<ArchiveScanner, IWarDeployableMetadata>();
        for (IArchive archive : archiveList) {
            // no lib deployable so call getDeployable will return null
            WarDeployableMetadataConfigurator warArchiveMetadataConfigurator =
                new WarDeployableMetadataConfigurator(null);
            ArchiveScanner archiveScanner = archiveScannerFactory.createArchiveScanner(archive);
            archiveScanner.addArchiveConfigurator(warArchiveMetadataConfigurator);
            scanners.put(archiveScanner, warArchiveMetadataConfigurator.getWarDeployableMetadata());
        }
    }

    /**
     * Try to find extern metadata.
     * @param warDeployableMetadata the war deployable
     * @param className the class name
     */
    public void attachClassMetadata(final WarDeployableMetadata warDeployableMetadata, final String className) {
        IWarClassMetadata warClassMetadata = classMetadata.get(className);
        if (warClassMetadata == null) {
            List<String> classNameList = new ArrayList<String>();
            classNameList.add(className);
            Iterator<Entry<ArchiveScanner, IWarDeployableMetadata>> entries = scanners.entrySet().iterator();
            while (warClassMetadata == null && entries.hasNext()) {
                Entry<ArchiveScanner, IWarDeployableMetadata> entry = entries.next();
                try {
                    // TODO gaellalire : add method scanClassName (instead of scan collection)
                    // which returns a boolean
                    entry.getKey().scanCollection(classNameList);
                    warClassMetadata = entry.getValue().getWarClassMetadata(className);
                } catch (ScanException e) {
                    LOGGER.debug("error when scan collection", e);
                }
            }
        }
        if (warClassMetadata != null) {
            classMetadata.put(className, warClassMetadata);
            try {
                warClassMetadata.attachClone(warDeployableMetadata);
                warDeployableMetadata.addWarClassMetadata(warClassMetadata);
            } catch (CloneNotSupportedException e) {
                LOGGER.debug("Can not add metadata", e);
            }
        }
    }

    /**
     * Stop all archive scanner.
     */
    public void close() {
        for (ArchiveScanner archiveScanner : scanners.keySet()) {
            archiveScanner.endScan();
        }
    }

}
